###### Fig 3.17 VaR using SMC
###### Written by Dr Jinguo Gong <jinguogong@swufe.edu.cn>

sp500=read.table("sp500.dat")
gs=read.table("goldmanSacks.dat")
ford=read.table("ford.dat")
r.sp500=100*diff(log(sp500[,6]))
r.gs=100*diff(log(gs[,6]))
r.ford=100*diff(log(ford[,6]))

D <- read.table("sp500.dates")
t <- D[,1]; D <- strptime(t, "%Y-%m-%d"); D=D[3:659]
alpha1=0.05;alpha2=0.025
i1=as.integer(alpha1*500)
i2=as.integer(alpha2*500)


###### Sequential Monte Carlo 
library(pomp)
SV.proc.sim <- function (x, t, params,delta.t, ...) {
     eps=rnorm(1,0,sqrt(exp(params["logsigw2"])))
     b1=(exp(params["b1_tran"])-1)/(exp(params["b1_tran"])+1)
     xnew=params["c"]+b1*x["H"]+eps 
     names(xnew)="H"
     return(xnew)
}

SV.meas.sim=function(x,t,params,...){
  eps=log(rnorm(1)^2+0.0000000001)
  y=x["H"]+eps
  names(y)="Y"
  return(y)
}

SV.meas.dens=function(y,x,t,params,log,...){
     temp=y["Y"]-x["H"]
     f=exp(-(exp(temp)-temp)/2)/sqrt(2*pi)
     return(f)
    }

####### Part 1: Create pomp object of SP500

p=4*657
T <- matrix(1:p, ncol=4, nrow=657)

for(n in 503:656) { 
	T[(n+1),1]=r.sp500[n+1]  # value to be predicted
	X <- r.sp500[(n-500+1):n] # using the previous 500 data points
      y=as.matrix(log(X^2))
      colnames(y) = "Y"
      num=length(y)
      h.inv=log(var(X))

     SV_sp=pomp(
        data=data.frame(
          time=1:num,
          Y=y
        ),
        times="time",
        t0=0,

       rprocess=discrete.time.sim(
            step.fun=SV.proc.sim,
            delta.t=1
            ),   
       rmeasure=SV.meas.sim,
       dmeasure=SV.meas.dens
     ) 


      out_sp <- nlf(
         SV_sp,
         start=c(b1_tran=0.3,c=-0.2,logsigw2=log(1.4),H.0=h.inv),
         transform.params=FALSE,
         est=c("b1_tran","c","logsigw2"),
         lags=c(1,2),
         seed=1234567,
         method="Nelder-Mead",
         trace=4,
         skip.se=FALSE,
         nasymp=5000
        )
      fit_sp <-out_sp$params[c("b1_tran","c","logsigw2")]
      pf_sp <- pfilter(SV_sp,params=c(fit_sp,H.0=h.inv),Np=1000, filter.mean=TRUE)

      xx=(exp(fit_sp[1])-1)/(exp(fit_sp[1])+1)
      pred.sig=fit_sp[2]+ xx*filter.mean(pf_sp)[500]
      T[(n+1),2]=as.vector(exp(pred.sig/2))
    
      Res=X/exp(filter.mean(pf_sp)/2)
      Res=sort(as.vector(Res))
	T[(n+1),3]=Res[i1] # alpha1-th quantile of residuals
	T[(n+1),4]=Res[i2] # alpha2-th quantile of residuals
}	





####### Part 2: Create pomp object of goldmanSacks

p=4*657
T_gs<- matrix(1:p, ncol=4, nrow=657)

for(n in 503:656) { 
	T_gs[(n+1),1]=r.gs[n+1]  # value to be predicted
	X <- r.gs[(n-500+1):n] # using the previous 500 data points
      y=as.matrix(log(X^2))
      colnames(y) = "Y"
      num=length(y)

     SV_sp=pomp(
        data=data.frame(
          time=1:num,
          Y=y
        ),
        times="time",
        t0=0,

       rprocess=discrete.time.sim(
            step.fun=SV.proc.sim,
            delta.t=1
            ),   
       rmeasure=SV.meas.sim,
       dmeasure=SV.meas.dens
     ) 

      out_sp <- nlf(
         SV_sp,
         start=c(b1_tran=0.3,c=-0.2,logsigw2=log(1.4),H.0=0.01),
         transform.params=FALSE,
         est=c("b1_tran","c","logsigw2"),
         lags=c(1,2),
         seed=1234,
         method="Nelder-Mead",
         trace=4,
         skip.se=FALSE,
         nasymp=5000
        )
      fit_sp <-out_sp$params[c("b1_tran","c","logsigw2")]
      pf_sp <- pfilter(SV_sp,params=c(fit_sp,H.0=0.01),Np=1000, filter.mean=TRUE)

      xx=(exp(fit_sp[1])-1)/(exp(fit_sp[1])+1)
      pred.sig=fit_sp[2]+ xx*filter.mean(pf_sp)[500]
      T_gs[(n+1),2]=as.vector(exp(pred.sig/2))

      Res=X/exp(filter.mean(pf_sp)/2)
      Res=sort(as.vector(Res))
	T_gs[(n+1),3]=Res[i1] # alpha1-th quantile of residuals
	T_gs[(n+1),4]=Res[i2] # alpha2-th quantile of residuals
}	

####### Part 3: Create pomp object of ford
n1=length(r.ford[r.ford!=0])
p=4*n1
T_ford<- matrix(1:p, ncol=4, nrow=n1)
r.ford=r.ford[r.ford!=0]

for(n in 548:(n1-1)) { 
	T_ford[(n+1),1]=r.ford[n+1]  # value to be predicted
	X <- r.ford[(n-500+1):n] # using the previous 500 data points
      y=as.matrix(log(X^2))
      colnames(y) = "Y"
      num=length(y)
       h.inv=log(var(X))

     SV_sp=pomp(
        data=data.frame(
          time=1:num,
          Y=y
        ),
        times="time",
        t0=0,

       rprocess=discrete.time.sim(
            step.fun=SV.proc.sim,
            delta.t=1
            ),   
       rmeasure=SV.meas.sim,
       dmeasure=SV.meas.dens
     ) 

      out_sp <- nlf(
         SV_sp,
         start=c(b1_tran=0.3,c=-0.2,logsigw2=log(1.4),H.0=h.inv),
         transform.params=FALSE,
         est=c("b1_tran","c","logsigw2"),
         lags=c(1,2),
         seed=1234,
         method="Nelder-Mead",
         trace=4,
         skip.se=FALSE,
         nasymp=5000
        )
      fit_sp <-out_sp$params[c("b1_tran","c","logsigw2")]
      pf_sp <- pfilter(SV_sp,params=c(fit_sp,H.0=h.inv),Np=1000, filter.mean=TRUE)

      xx=(exp(fit_sp[1])-1)/(exp(fit_sp[1])+1)
      pred.sig=fit_sp[2]+ xx*filter.mean(pf_sp)[500]
      T_ford[(n+1),2]=as.vector(exp(pred.sig/2))

      Res=X/exp(filter.mean(pf_sp)/2)
      Res=sort(as.vector(Res))
	T_ford[(n+1),3]=Res[i1] # alpha1-th quantile of residuals
	T_ford[(n+1),4]=Res[i2] # alpha2-th quantile of residuals
}	

########## Output

ps.options(horizontal=F, width=4.8, height=6, pointsize=9)
postscript("Fig317.ps", onefile=F, print=F)
pdf("Fig317.pdf", onefile=F)

par(mar=c(3,3,3,1),mfrow=c(3,1), mgp=c(1.8,0.6,0))
plot(r.sp500[504:657], type='l', xlab='', ylab='', ylim=c(-3.9,2.5),
          xaxt = "n", main='Daily returns of S&P500 index in 2012')
lines(T[504:657,2]*T[504:657,3], col="blue") #5%
lines(T[504:657,2]*T[504:657,4], col="darkgreen") #2.5%
lines(-T[504:657,2]*1.96, col="red")      # 2.5%
lines( -T[504:657,2]*1.64, col="orange")   # 5%
axis(1,at=c(1,41,83,126),c("Jan","Mar","May","Jul") )

print(table((T[504:657,2]*T[504:657,3])<T[504:657,1])/154)
print(table((T[504:657,2]*T[504:657,4])<T[504:657,1])/154)
print(table((-T[504:657,2]*1.64)<T[504:657,1])/154)
print(table((-T[504:657,2]*1.96)<T[504:657,1])/154)

plot(r.gs[504:657], type='l', xlab='', ylab='', ylim=c(-7.5,6), 
          xaxt = "n", main='Daily returns of S&P500 index in 2012')
lines(T_gs[504:657,2]*T_gs[504:657,3], col="blue") #5%
lines(T_gs[504:657,2]*T_gs[504:657,4], col="darkgreen") #2.5%
lines(-T_gs[504:657,2]*1.96, col="red")      # 2.5%
lines( -T_gs[504:657,2]*1.64, col="orange")   # 5%
axis(1,at=c(1,41,83,126),c("Jan","Mar","May","Jul") )

print(table((T_gs[504:657,2]*T_gs[504:657,4])<T_gs[504:657,1])/154)
print(table((T_gs[504:657,2]*T_gs[504:657,3])<T_gs[504:657,1])/154)
print(table((-T_gs[504:657,2]*1.64)<T_gs[504:657,1])/154)
print(table((-T_gs[504:657,2]*1.96)<T_gs[504:657,1])/154)


D1=D[r.ford!=0]
plot(r.ford[r.ford!=0][504:n1], type='l', xlab='',
        xaxt = "n", ylab='', ylim=c(-6.0,4), main='Daily returns of Ford stock in 2012')
lines(T_ford[504:n1,2]*T_ford[504:n1,3], col="blue")
lines(T_ford[504:n1,2]*T_ford[504:n1,4], col="darkgreen")
lines( -T_ford[504:n1,2]*1.96, col="red")
lines( -T_ford[504:n1,2]*1.64, col="orange")
axis(1,at=c(1,37,75,113),c("Jan","Mar","May","Jul") )

print(table((T_ford[504:n1,2]*T_ford[504:n1,4])<T_ford[504:n1,1])/139)
print(table((T_ford[504:n1,2]*T_ford[504:n1,3])<T_ford[504:n1,1])/139)
print(table((-T_ford[504:n1,2]*1.64)<T_ford[504:n1,1])/139)
print(table((-T_ford[504:n1,2]*1.96)<T_ford[504:n1,1])/139)

dev.off()

